package matrices;

import java.beans.ConstructorProperties;
import java.io.*;
import java.util.ArrayList;
import java.util.StringTokenizer;

public class Matrix implements Cloneable {
    private double[][] array;

    public Matrix() {
        array = new double[0][0];
    }

    public Matrix(int m, int n) {
        array = new double[m][n];
    }

    @ConstructorProperties({"array"})
    public Matrix(double[][] array) {
        this.array = ArrayUtil.clone(array);
    }

    public Matrix(double[] array, int cols) {
        this.array = ArrayUtil.oneToTwo(array, cols);
    }

    public Matrix(Matrix matrix) {
        this.array = ArrayUtil.clone(matrix.array);
    }

    public Matrix(String path) {
        array = read(path);
    }

    /**
     * Ustawia wszystkie komórki macierzy na podaną wartość
     *
     * @param value - wartość do wstawienia
     */
    public void setToValue(double value) {
        for (int i = 0; i < getRowCount(); i++) {
            for (int j = 0; j < getColCount(); j++) {
                array[i][j] = value;
            }
        }
    }

    /**
     * @return zwraca liczbę rzędów w macierzy
     */
    public int getRowCount() {
        return array.length;
    }

    /**
     * @return zwraca liczbę kolumn w macierzy
     */
    public int getColCount() {
        return array[0].length;
    }

    /**
     * Drukuje tę macierz na konsoli
     */
    public void printToConsole() {
        for (int i = 0; i < getRowCount(); i++) {
            for (int j = 0; j < getColCount(); j++) {
                System.out.print(array[i][j] + " ");
            }
            System.out.println();
        }
    }

    public void printToConsole(int dec) {
        for (int i = 0; i < getRowCount(); i++) {
            for (int j = 0; j < getColCount(); j++) {
                System.out.print(MathUtil.roundToDecimal(array[i][j], dec)
                        + " ");
            }
            System.out.println();
        }
    }

    /**
     * Zapisuje macierz do podanego pliku. Wartości w rzędzie
     * oddzielone są spacją. Rząd kończy się średnikiem
     *
     * @param path - plik
     */
    public void write(String path) {
        try (PrintWriter pw = new PrintWriter(path)) {
            for (int i = 0; i < getRowCount(); i++) {
                for (int j = 0; j < getColCount() - 1; j++) {
                    pw.print(array[i][j] + " ");
                }
                pw.print(array[i][getColCount() - 1]);
                pw.print(";");
                pw.println();
            }
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        }
    }

    /**
     * Wczytuje macierz z pliku
     *
     * @param path - ścieżka do pliku z macierzą
     * @return zwraca wczytaną macierz
     */
    public double[][] read(String path) {
        double[][] tabl;
        ArrayList<String> al = new ArrayList<>();
        BufferedReader in;
        StringTokenizer st;
        int len;
        int len1;
        try {
            in = new BufferedReader(new FileReader(new File(path)));
            try {
                String s;
                while ((s = in.readLine()) != null) {
                    al.add(s);
                }
            } finally {
                in.close();
            }
        } catch (IOException ioe) {
            throw new RuntimeException(ioe);
        }
        st = new StringTokenizer(al.get(0), " ;");
        len = st.countTokens();
        len1 = al.size();
        tabl = new double[len1][len];
        for (int i = 0; i < len1; i++) {
            st = new StringTokenizer(al.get(i), " ;");
            int j = 0;
            while (st.hasMoreTokens()) {
                tabl[i][j] = Double.parseDouble(st.nextToken());
                j++;
            }
        }
        return tabl;
    }

    /**
     * Zwraca glęboką kopię tej macierzy
     */
    @Override
    public Matrix clone() throws CloneNotSupportedException {
        Matrix cl = (Matrix) super.clone();
        cl.array = ArrayUtil.clone(array);
        return cl;
    }
}
